#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>

#define MAX_LEN 20
#define LETTER_WIDTH 5

char msg[MAX_LEN] = "ADRIANA";
int msg_len = 7;

const int letters[][5] = 
{ 
 {0x7e, 0x09, 0x09, 0x09, 0x7e}, // A
 {0x7f, 0x49, 0x49, 0x49, 0x36}, // B
 {0x3e, 0x41, 0x41, 0x41, 0x22}, // C
 {0x7f, 0x41, 0x41, 0x22, 0x1c}, // D
 {0x7f, 0x49, 0x49, 0x49, 0x49}, // E
 {0x7f, 0x09, 0x09, 0x09, 0x09}, // F
 {0x7e, 0x41, 0x49, 0x49, 0x38}, // G
 {0x7f, 0x08, 0x08, 0x08, 0x7f}, // H
 {0x00, 0x41, 0x7f, 0x41, 0x00}, // I
 {0x30, 0x40, 0x41, 0x7f, 0x01}, // J
 {0x7f, 0x08, 0x14, 0x22, 0x41}, // K
 {0x7f, 0x40, 0x40, 0x40, 0x40}, // L
 {0x7f, 0x02, 0x04, 0x02, 0x7f}, // M
 {0x7f, 0x02, 0x0c, 0x10, 0x7f}, // N
 {0x3e, 0x41, 0x41, 0x41, 0x3e}, // O
 {0x7f, 0x49, 0x49, 0x49, 0x36}, // B
 {0x1e, 0x21, 0x21, 0x21, 0x5e}, // Q
 {0x7f, 0x09, 0x19, 0x29, 0x46}, // R
 {0x26, 0x49, 0x49, 0x49, 0x32}, // S
 {0x01, 0x01, 0x7f, 0x01, 0x01}, // T
 {0x3f, 0x40, 0x40, 0x40, 0x3f}, // U
 {0x07, 0x38, 0x40, 0x38, 0x07}, // V
 {0x3f, 0x40, 0x3f, 0x40, 0x3f}, // W
 {0x63, 0x14, 0x08, 0x14, 0x63}, // X
 {0x07, 0x08, 0x70, 0x08, 0x07}, // Y
 {0x61, 0x51, 0x49, 0x45, 0x43}, // Z  
 {0x00, 0x00, 0x00, 0x00, 0x00}  // space
};

volatile unsigned char text[512];
volatile unsigned int text_len = 0;
volatile unsigned int text_idx = 0;

void start(void) 
{
	unsigned int i, j;
	for (i = 0; i < msg_len; i++) {
		for (j = 0; j < LETTER_WIDTH; j++) {
			text[text_len] = letters[msg[i] - 'A'][j];
			text_len = text_len + 1;
		}
		for (j = 0; j < LETTER_WIDTH; j++) {
			text[text_len] = 0;
			text_len = text_len + 1;
		}
	}
	for (i = 0; i < msg_len; i++) {
		for (j = 0; j < LETTER_WIDTH; j++) {
			text[text_len] = 0;
			text_len = text_len + 1;
		}
	}
}

void advance(void)
{
	if (text_idx < text_len) {
		PORTA = 255 ^ text[text_idx++];
		PORTB = 255 ^ text[text_idx++];
	} else {
		text_idx = 0;
	}
}

ISR(TIMER1_COMPA_vect)
{
	advance();
}

int main() 
{
	int i;
	
	sei();
	
	text_idx = 0;
	text_len = 0;
	
	DDRA = 0xFF;
	DDRB = 0xFF;
	
	start();
	
	//initializare timer
	OCR1A = 15625;

	TCCR1A = 0;
	TCCR1B = (1<<WGM12) | _BV(CS12) | _BV(CS10);
	
	//TIMSK1 |= _BV(OCIE1A);
	ICR1 = 65535;

    TIMSK1 |=  ( 1<< OCIE1A);
	
	while (1) {
		for (i = 0; i < 8; i++) {
			PORTA &= ~(1 << i); // aprins
			PORTB &= ~(1 << i);
			_delay_ms(1000);
			PORTA |= (1 << i); // stins	
			PORTB |= (1 << i);
		}
	}
	
	return 0;
}

